/**
 * HTLM header toolbar class
 * 
 * Todo:
 *  - integrate url picker.
 *  - integrate advanced edit (make style attr editable only where applicable)
 *  - integrate style edit on <link rel="stylesheet" /> and <style>
 *  - remove tag
 *  - add plain text css editor for quick access/quick check
 *  - add rdf data files for pulldown option values/labels. make these editable (rememberable) where applicable (ie. allow updates to the rdf file)
 *  - 16x16px icons alla classic theme (CI/CD)
 *  - some tags/attribute constalations may appear only once inside head 
 *    (like title, or base href=""). check for duplicates before insert 
 *    dialog is opened (depending on tag/attribute constalation) and 
 *    before actually inserting (depending on attributes)
 *  - integrate Moz toolbar configuration (add / remove buttons)
 *  - drag move items.
 *  - check for compliance with a bunch of other dtd's like, html 1-4, xhtml 1.
 *  - credits (gimp icons at least!)
 *  - integrate into about dialog.
 *  - xpi
 */

//var currentTag = null;
var htmlHeader = {
	currentTag: null,
	initialized: false,
	updateInProgress: false,
	toolbar : null,
	selectedEditorIndex: 0,
	tabEditor: null,
	tabBox: null,
	init : function() {
		this.tabEditor = document.getElementById("tabeditor");
		this.tabBox = this.tabEditor.mTabbox;
		if (!this.tabEditor || this.initialized)
			return;
		
		this.tabBox.addEventListener("focus", htmlHeader.evDocumentChange, true);
		this.tabEditor.addEventListener("load", htmlHeader.evDocumentChange, true);
		document.getElementById("content-source").addEventListener("focus", htmlHeader.evDocumentChange, true);
		htmlHeader.selectedEditorIndex = document.getElementById("tabeditor").selectedIndex;

		this.initialized = true;
	},
	
	evDocumentChange : function(ev) {
		//htmlMetaConsole.consoleOut(ev.currentTarget.id + "|" + ev.target.id +": " + ev.type);

		// 2 things to do: 
		//   1. if the target is source-editor, disable toolbar
		//   2. else update buttons deleayed and enable toolbar
		
		if (ev.currentTarget.id == "content-source" && ev.type == "focus") {
			document.getElementById("HtmlHeaderToolbox").setAttribute("collapsed", true);
		} else if (!htmlHeader.updateInProgress) {
			//htmlMetaConsole.consoleOut("Delayed update");
			htmlHeader.updateInProgress = true;
			document.getElementById("HtmlHeaderToolbox").setAttribute("collapsed", false);
			setTimeout('htmlHeader.updateToolbar()', 350);
		}
			
		return true;
	},
	
	getHeadChildren : function() {
		// find header tag
		var header   = this.document().getElementsByTagName("HEAD").item(0);
		return header.childNodes; // ok i am lazy
	},
	
	document : function() {
		return GetCurrentEditor().document;
	}, 

	updateToolbar : function() {
		htmlHeader.updateInProgress = false;

//		htmlMetaConsole.consoleOut("== updating");
		this.clearButtons();
		
		if (!this.toolbar)
			this.toolbar = document.getElementById("HtmlHeaderToolbar");
		
		// find header tag
		var tagName = '';
		var header   = this.header();
		var children = header.childNodes; // ok i am lazy
//		htmlMetaConsole.consoleOut("number of tags in HEAD: " + children.length);
		for(var i=0; i<children.length; i++) {
			e=children.item(i);
			if (e.nodeType == TEXT_NODE /* 3 */ )
				continue; // skip whitespace 
//			htmlMetaConsole.consoleOut(e.tagName + " " + e.nodeType);
			
			if (e.nodeType == COMMENT_NODE)
				tagName = "comment";
			else if (e.tagName)
				tagName = e.tagName.toLowerCase();
			else tagName = '';
			
			// if this node is not a comment or one of the 
			// implemented tags, skip it
			//
			// FIXME: use if (tagName in SUPPORTED_TAGS) 
			if (tagName != 'comment' && tagName != 'link' && tagName != 'meta' && 
			    tagName != 'script' && tagName != 'style' && tagName != 'title' && 
			    tagName != 'base')
				continue;
			
			this.addButton(e);
		}
		
//		htmlMetaConsole.consoleOut("== updating, done.");
	},
	
	header : function () {
		return this.document().getElementsByTagName("HEAD").item(0);
	},
	
	addButton : function(headerTag) {

		// figure the image name
		var tagName = '';
		var imgName = "default";
		if(headerTag.tagName)
			tagName = headerTag.tagName.toLowerCase();
		if (headerTag.nodeType == COMMENT_NODE)
			imgName = "comment";
		// FIXME: use if (tagName in SUPPORTED_TAGS) 
		else if (tagName == 'link' || tagName == 'meta' || tagName == 'script' || 
		         tagName == 'style' || tagName == 'title' || tagName == 'base')
			imgName = tagName;

		var newBtn = document.createElement("toolbarbutton");
		var lbl = document.createElement("label");
		var img = document.createElement("image");
		newBtn.appendChild(img);
		newBtn.appendChild(lbl);
		newBtn.setAttribute("id", headerTag.id);
		img.setAttribute("src", "chrome://htmlheader/content/images/icons/"+imgName+"-16.png");
		
		var lblValue = "<!--";
		
		if (headerTag.nodeType != COMMENT_NODE)
			lblValue = headerTag.tagName.toUpperCase()
		
		lbl.setAttribute("value", lblValue);
		newBtn.label = headerTag.tagName;
		newBtn.associatedElement = headerTag;

		this.toolbar.appendChild(newBtn);

		newBtn.addEventListener("command", htmlHeader.editTag, true);
		newBtn.assiciatedElement = headerTag;
	},

	clearButtons : function() {
		// alert("htmlHeader.clearButtons() : not yet implemented!");
		var buttonStart = document.getElementById("htmlHeaderButtonsStart");
		while( (e=buttonStart.nextSibling) ) {
			buttonStart.parentNode.removeChild(e);
		}
		return true;
	},
	
	editTag : function(e) {
	
		htmlHeader.currentTag = e.target.associatedElement;
//		var options = 'all,dialog=yes';
		var arg1 = htmlHeader.currentTag.cloneNode(true);

		htmlHeader.editTagDialog(arg1);
	},
	
	editTagDialog: function(el) {
		var url = "chrome://htmlheader/content/editTag-";
		var options = 'all,dialog=yes,modal=yes';
		var tag = "";
		if (htmlHeader.currentTag.nodeType != COMMENT_NODE)
			tag = htmlHeader.currentTag.tagName.toLowerCase();
		else
			tag = "comment";
			
		url += tag+".xul";
		window.openDialog(url, '_blank', options, el);
	},
	
	addTag : function(el) {
		if (el.label == "Comment")
			var tag = document.createComment("");
		else
			var tag = document.createElement(el.label.toLowerCase());

		tag.createNew = true;
		
		htmlHeader.currentTag = tag;
		htmlHeader.editTagDialog(tag);
	},
	
	insertTag : function(attr, tagBody) {
		
		var head = this.header();
		//htmlMetaConsole.dumpObject(currentTag);
		if (htmlHeader.currentTag.nodeType == COMMENT_NODE) {
			var newTag = document.createComment(htmlHeader.tagBody);
			if (tagBody) 
				newTag.nodeValue = tagBody;
		} else {
			var newTag = document.createElement(htmlHeader.currentTag.tagName.toLowerCase());
			
			for (e in attr) {
//				p(e);
				newTag.setAttribute(e, attr[e]);
			}
			
			if (tagBody) 
				newTag.appendChild(document.createTextNode(tagBody));
		}

		head.appendChild(newTag);
		var ed = GetCurrentEditor();
		ed.incrementModificationCount(1);
		
		// check if the tab title has changed
		if(htmlHeader.currentTag.tagName && 
		   htmlHeader.currentTag.tagName.toLowerCase() == "title") {
//			ed.markNodeDirty(newTag);
			ed.setDocumentTitle(tagBody);
//			UpdateWindowTitle();
		}
		
		
		// last but not least, update the toolbar
		this.updateToolbar();
		return true;
	},
	
	replaceTag : function(attr, tagBody) {
		var head = this.header();
		//htmlMetaConsole.dumpObject(currentTag);
		if (htmlHeader.currentTag.nodeType == COMMENT_NODE) {
			var newTag = document.createComment(htmlHeader.tagBody);
			if (tagBody) 
				newTag.nodeValue = tagBody;
		} else {
			var newTag = document.createElement(htmlHeader.currentTag.tagName.toLowerCase());
			
			for (e in attr) {
//				p(e);
				newTag.setAttribute(e, attr[e]);
			}
			
			if (tagBody) 
				newTag.appendChild(document.createTextNode(tagBody));
		}

		head.replaceChild(newTag, htmlHeader.currentTag);
		var ed = GetCurrentEditor();
		ed.incrementModificationCount(1);
		
		// check if the tab title has changed
		if(htmlHeader.currentTag.tagName && 
		   htmlHeader.currentTag.tagName.toLowerCase() == "title") {
//			ed.markNodeDirty(newTag);
			ed.setDocumentTitle(tagBody);
//			UpdateWindowTitle();
		}
		
		
		// last but not least, update the toolbar
		this.updateToolbar();
		return true;
	}
	
};

//window.addEventListener("load", htmlHeader.init(), false);


var htmlMetaConsole = {
	inspectDocument : function(doc) {
		window.openDialog('chrome://inspector/content/', 
		                  '_blank', 'all,dialog=no', doc);
	},
	
	dumpObject : function(o) {
		var out = "== BEGIN OBJECT ==\n";
		for (e in o) {
			out += e;
			try {
				if (typeof(o[e]) == "function")
					out += "()";
				else
					out += ": " + o[e];
			} catch(err) {
				out += ": Error: " + err;
			}
			out += '\n';
		}
		this.consoleOut(out + "== END OBJECT ==\n\n");
	},
	
	consoleOut : function(txt) {
		document.getElementById("htmlHeaderConsole").value += txt + "\n";
	},
	
	show : function() {
		var sidebar = document.getElementById("sidebar-box").collapsed;
		document.getElementById("sidebar-box").collapsed = (!sidebar) ? true : false;
	}

};

function p(txt) { htmlMetaConsole.consoleOut(txt); }
function d(obj) { htmlMetaConsole.dumpObject(obj); }

/*var metaHeader = {
	editor     : document.getElementById("content-source"),
	document   : null,
	toolbar    : null,
	
	init: function() {
		this.document = document.getElementById("content-source").contentDocument;
		this.toolbar  = document.getElementById("HtmlHeaderToolbar");

		// clear it
		this.clearButtons();
		
		// find the HEAD tag of our document
		var html   = this.document.childNodes.item(0);
		var header = this.document.getElementsByTagName("HEAD").item(0);
		htmlMetaConsole.consoleOut(this.document.
			getElementsByTagName("BODY").item(0).id);
		if (!header) // no HEAD tag, bail.
			return false;
		
		// prototype button
		var btn = this.newButton();
		
		// loop over header entries and insert them into the button bar
		var i = 0; 
		var e = null;
		while (e = header.childNodes.item(i++) ) {
			var newBtn = btn.cloneNode(true);
			newBtn.setAttribute("id", "headerBtn"+i);
			newBtn.childNodes.item(0).setAttribute("value", i + 
				" " +e.childNodes.item(0).value);
			this.toolbar.appendChild(newBtn);
		}
		
	},
	
	newButton : function() {
		// prototype button
		var btn = document.createElement("toolbarbutton");
		var lbl = document.createElement("label");
		btn.appendChild(lbl);
		return btn;
	},
	
	editTag: function(target) {
		alert(target.id);
	}
}; */

